package core.entities.timemanager;

import core.api.models.timemanager.tasks.TaskFieldInfo;

import javax.persistence.*;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;

@Entity
@Table(name = "tasks")
public class Task {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long taskID;

    @ManyToOne
    @JoinColumn(name = "taskgroup_id")
    private Taskgroup taskgroup;
    private String taskName;

    private LocalDate startDate;

    private LocalDate deadline;

    private int eta;

    private boolean finished;

    private boolean finishable;

    @OneToMany(mappedBy = "task", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    private List<AbstractSchedule> basicTaskSchedules;

    private int workTime;

    public Task() {
    }

    public Task(Taskgroup taskgroup, TaskFieldInfo taskFieldInfo) {
        this.taskgroup = taskgroup;
        this.taskName = taskFieldInfo.getTaskName();
        this.startDate = taskFieldInfo.getStartDate();
        this.deadline = taskFieldInfo.getDeadline();
        this.eta = taskFieldInfo.getEta();
        this.finished = false;
        this.finishable = taskFieldInfo.isFinishable();
        System.err.println(this.finishable);
    }

    public long getTaskID() {
        return taskID;
    }

    public Taskgroup getTaskgroup() {
        return taskgroup;
    }

    public void setTaskgroup(Taskgroup taskgroup) {
        this.taskgroup = taskgroup;
    }

    public String getTaskName() {
        return taskName;
    }

    public void setTaskName(String taskName) {
        this.taskName = taskName;
    }

    public LocalDate getStartDate() {
        return startDate;
    }

    public void setStartDate(LocalDate startDate) {
        this.startDate = startDate;
    }

    public LocalDate getDeadline() {
        return deadline;
    }

    public void setDeadline(LocalDate deadline) {
        this.deadline = deadline;
    }

    public int getEta() {
        return eta;
    }

    public void setEta(int eta) {
        this.eta = eta;
    }

    public void finish() {
        this.finished = !this.finished;
    }

    public boolean isFinished() {
        return finished;
    }

    public void setFinished(boolean finished) {
        this.finished = finished;
    }

    public int getWorkTime() {
        return workTime;
    }

    public void setWorkTime(int workTime) {
        this.workTime = workTime;
    }

    public void addWorkTime(int workTime) {
        this.workTime += workTime;
    }

    public void setTaskID(long taskID) {
        this.taskID = taskID;
    }

    public List<AbstractSchedule> getBasicTaskSchedules() {
        return basicTaskSchedules;
    }

    public void setBasicTaskSchedules(List<AbstractSchedule> basicTaskSchedules) {
        this.basicTaskSchedules = basicTaskSchedules;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Task task = (Task) o;
        return taskID == task.taskID && eta == task.eta && finished == task.finished && workTime == task.workTime && Objects.equals(taskgroup, task.taskgroup) && Objects.equals(taskName, task.taskName) && Objects.equals(startDate, task.startDate) && Objects.equals(deadline, task.deadline);
    }

    @Override
    public int hashCode() {
        return Objects.hash(taskID, taskgroup, taskName, startDate, deadline, eta, finished, workTime);
    }

    @Override
    public String toString() {
        return "Task{" +
                "taskID=" + taskID +
                ", taskgroup=" + taskgroup +
                ", taskName='" + taskName + '\'' +
                ", startDate=" + startDate +
                ", deadline=" + deadline +
                ", eta=" + eta +
                ", finished=" + finished +
                ", workTime=" + workTime +
                '}';
    }

    public boolean hasActiveSchedule() {
        for (AbstractSchedule schedule : getBasicTaskSchedules()) {
            if (schedule.isActive()) {
                return true;
            }
        }
        return false;
    }

    public List<AbstractSchedule> getStartableSchedules() {
        List<AbstractSchedule> abstractSchedules = new ArrayList<>();
        for(AbstractSchedule schedule : getBasicTaskSchedules()) {
            if(schedule.isStartable()) {
                abstractSchedules.add(schedule);
            }
        }
        return abstractSchedules;
    }

    public boolean isFinishable() {
        return finishable;
    }

    public void setFinishable(boolean finishable) {
        this.finishable = finishable;
    }
}
